@@ -7,7 +7,7 @@ use Exporter 'import';
 use ExtUtils::MakeMaker;
 use XS::Install::Payload;
 
-our $VERSION = '1.0.3';
+our $VERSION = '1.0.4';
 my $THIS_MODULE = 'XS::Install';
 
 our @EXPORT_OK = qw/write_makefile makemaker_args/;
@@ -61,11 +61,7 @@ sub makemaker_args {
     }
 
     $params{CONFIGURE_REQUIRES} ||= {};
-    $params{CONFIGURE_REQUIRES}{'ExtUtils::MakeMaker'} ||= '6.76';
-    
     $params{BUILD_REQUIRES} ||= {};
-    $params{BUILD_REQUIRES}{'ExtUtils::MakeMaker'} ||= '6.76';
-    $params{BUILD_REQUIRES}{'ExtUtils::ParseXS'}   ||= '3.24';
     
     $params{TEST_REQUIRES} ||= {};
     $params{TEST_REQUIRES}{'Test::Simple'} ||= '0.96';
@@ -160,9 +156,7 @@ sub makemaker_args {
     $params{CCFLAGS} = "$Config{ccflags} $params{CCFLAGS}" if $params{CCFLAGS};
     $params{OPTIMIZE} = "$Config{optimize} -O2 ".($params{OPTIMIZE}||'');
 
-    if (!$params{C} || !@{$params{C}} and !$params{OBJECT} || !@{$params{OBJECT}} and !$params{XS} || !scalar(keys %{$params{XS}})) {
-        delete $params{$_} for qw/C H OBJECT XS CCFLAGS LDFROM/;
-    }
+    delete @params{qw/C H OBJECT XS CCFLAGS LDFROM/} unless has_binary(\%params);
 
     process_test(\%params, $shared_libs_linking);
     
@@ -333,7 +327,8 @@ sub _apply_BIN_DEPS {
     
     return if $seen->{$module}++;
     
-    my $installed_version = XS::Install::Payload::module_version($module);
+    my $installed_version = XS::Install::Payload::module_version($module)
+        or die "[XS::Install] binary dependency '$module' must be installed to proceed\n";
     $params->{CONFIGURE_REQUIRES}{$module}  ||= $installed_version;
     $params->{PREREQ_PM}{$module}           ||= $installed_version;
     $params->{MODULE_INFO}{BIN_DEPS}{$module} = $installed_version;
@@ -351,7 +346,8 @@ sub _apply_BIN_DEPS {
         last;
     }    
     
-    my $info = XS::Install::Payload::module_info($module);
+    my $info = XS::Install::Payload::module_info($module)
+        or die "[XS::Install] this module wants '$module' as a binary dependence, however '$module' doesn't provide any binary interface\n";
     
     if ($info->{INCLUDE}) {
         my $incdir = XS::Install::Payload::include_dir($module);
@@ -441,7 +437,8 @@ sub process_BIN_SHARE {
     _module_info_write($infopath, $bin_share);
     
     my $pm = $params->{PM} ||= {};
-    $pm->{$infopath} = '$(INST_ARCHLIB)/$(FULLEXT).x/info';
+    my $where = has_binary($params) ? '$(INST_ARCHLIB)' : '$(INST_LIB)';
+    $pm->{$infopath} = $where.'/$(FULLEXT).x/info';
 }
 
 sub attach_BIN_DEPENDENT {
@@ -502,7 +499,7 @@ sub process_test {
         'TEST_XS_FILES = '.join(' ', keys %{$tp->{XS}}),
         '$(TEST_XS_FILES) :: $(FIRST_MAKEFILE) '.join(' ', @xsi_files).'; $(TOUCH) $(TEST_XS_FILES)',
         "TEST_INST_DYNAMIC = $dlpath",
-        'TEST_LDFROM = $(TEST_OBJECT) '.$shared_libs_linking,
+        'TEST_LDFROM = $(TEST_OBJECT) '.($shared_libs_linking||''),
         '$(TEST_INST_DYNAMIC) : $(TEST_OBJECT)'."\n".
             "\t".'$(RM_F) $@'."\n".
             "\t".'$(LD) $(LDDLFLAGS) $(TEST_LDFROM) $(OTHERLDFLAGS) -o $@ $(INST_DYNAMIC_FIX)'."\n".
@@ -514,6 +511,14 @@ sub process_test {
     delete @$tp{qw/SRC C H XS OBJECT CPLUS/};
 }
 
+sub has_binary {
+    my $params = shift;
+    return 1 if $params->{C} && @{$params->{C}};
+    return 1 if $params->{OBJECT} && @{$params->{OBJECT}};
+    return 1 if $params->{XS} && scalar(keys %{$params->{XS}});
+    return;
+}
+
 sub cmd_sync_bin_deps {
     my $myself = shift @ARGV;
     my @modules = @ARGV;
@@ -738,11 +743,7 @@ sub _module_info_write {
             chmod $mode, $file;
         }
     }
-    open my $fh, '>', $file or do {
-        warn "Cannot open $file for writing: $!, \033[1;31mbinary deps info won't be synced!\033[0m\n";
-        sleep 2;
-        return;
-    };
+    open my $fh, '>', $file or die "Cannot open $file for writing: $!, binary data could not be written\n";
     print $fh $content;
     close $fh;
     
